# WARIO: Weighted fAmilies of contact maps to chaRacterize conformational ensembles of (highly-)flexIble prOteins

WARIO is a tool for the structural characterization of highly-flexible proteins. WARIO takes as input a conformational ensemble (e.g. generated form molecular dynamics simulations or other sampling methods) and represents it as a weighted family of contact maps. Contact is redefined by a continuous function, taking values in $[0,1]$, that incorporates the relative orientation of the interacting residues as well as the sequence information. Then, the featured data is embedded into a 10-dimensional [UMAP](https://umap-learn.readthedocs.io/en/latest/index.html) space and clustered using the [HDBSCAN](https://hdbscan.readthedocs.io/en/latest/#) algorithm. Finally, the average values of the contact function across each cluster conformation are represented as cluster-specific contact maps. The maps are assigned with a weight given by the cluster occupancy. The following figure illustrates the main steps of the pipeline.

<br />
<br />
<br />
<div align="center">
<img src="images/pipeline.png"  width="800" height="400">
</div>
<br />
<br />
<br />

## The paper

The complete and detailed presentation of the method is available in the WARIO paper: [https://doi.org/10.21203/rs.3.rs-4149901/v1](https://doi.org/10.21203/rs.3.rs-4149901/v1).


## Installing WARIO

WARIO and its required dependencies can be automatically installed if Python >=3.8 is available. We recommend to perform the installation inside a [Python virtual environment](https://packaging.python.org/en/latest/guides/installing-using-pip-and-virtual-environments/). It can be created as follows
```
python3 -m venv pythonEnv
source pythonEnv/bin/activate
```
Then, WARIO is installed with
```
pip install -U pip
pip install git+https://gitlab.laas.fr/moma/WARIO.git
```
Once the installation is completed, the command
```
wario-notebooks
```
opens the ready-to-use jupyter notebook.

The installation procedure works correctly with recent versions of Linux (Ubuntu 20.04 and 22.04) and MacOS (Sonoma 14.4.1) operating systems. Typical install time on a normal desktop computer is around 5 minutes. If you encounter any trouble to install WARIO, please file an [issue](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/issues) or [contact us](mailto:javier.gonzalezdelgado@mcgill.ca).

## Running WARIO

To run WARIO to characterize an ensemble, the user can directly execute the [contact_clustering](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/blob/main/wario/contact_clustering.ipynb) notebook, which contains the detailed pipeline and allows a step-by-step implementation of the tool. The notebook takes a conformational ensemble as input and returns the weighted family of contact maps that characterizes the ensemble. The data featurization is performed at a first stage, allowing the user to adjust the resolution of the clustering algorithm afterwards. Then, clustering and post-processing are performed, and the results are displayed and saved. 

### Input data

Ensembles can be given as input in several of the most common data formats. WARIO accepts one .xtc file together with a topology file in any format admitted by [MDTraj](https://mdtraj.org/), one multiframe .pdb file, or a folder containing one .pdb file per conformation. Users can also choose to characterize sequence segments instead of the entire sequence. Details are provided in [contact_clustering](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/blob/main/wario/contact_clustering.ipynb). It should be noted that the current implementation of WARIO requires an all-atom representation of the protein backbone. 

### Output and results

The main output of WARIO is given through a weighted set of $\omega$-contact maps depicting the interaction patterns that characterize each cluster. Plots with cluster-specific DSSP propensities and the average radius of gyration are also provided. The notebook allows to export the clusters of conformations in the same format as the input ensemble. These files can be used for further analysis of the retrieved contact patterns and for the calculation of other structural descriptors corresponding to the practitioner's needs.


## DEMO (data example)

To test WARIO, we provide a dataset corresponding to the conformational ensemble of a small peptide, P-113, obtained from [MD simulations](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/tree/main/wario/P113). Data was extracted from [(Jephthah _et al._, 2021)](https://doi.org/10.1021/acs.jctc.1c00408), where details on the simulation and the force-field can be found. The P-113 ensemble contains $n=100001$ conformations for a sequence of length $L=12$. To run WARIO on this data, the user can uncomment the lines

```
#ensemble_folder = "/".join([path_to_notebook,'P113']) # Path to the folder containing trajectory files
#ensemble_name = "P113" # Name the ensemble
```
in the second code cell of the [contact_clustering](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/blob/main/wario/contact_clustering.ipynb) notebook, and proceed following the usual guidelines. Then, results will be automatically saved in the [P113 folder](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/tree/main/wario/P113). The typical running time for this data example is around 30 minutes in a normal desktop computer using 5 threads. 

By setting the clustering precision at the default level (i.e. minimum cluster size = 1% of the total number of conformations), WARIO partitions the P-113 ensemble into 12 clusters and classifies the 93.04% of conformations. The ensemble is characterized by the following family of weighted cluster-specific contact maps:

<br />
<br />
<div align="center">
<img src="demo/P113_0.png"  width="180" height="135">
<img src="demo/P113_1.png"  width="180" height="135">
<img src="demo/P113_2.png"  width="180" height="135">
<img src="demo/P113_3.png"  width="180" height="135">
<img src="demo/P113_4.png"  width="180" height="135">
<img src="demo/P113_5.png"  width="180" height="135">
</div>
<div align="center">
<img src="demo/P113_6.png"  width="180" height="135">
<img src="demo/P113_7.png"  width="180" height="135">
<img src="demo/P113_8.png"  width="180" height="135">
<img src="demo/P113_9.png"  width="180" height="135">
<img src="demo/P113_10.png"  width="180" height="135">
<img src="demo/P113_11.png"  width="180" height="135">
</div>
<br />
<br />

Structural properties such as the average DSSP propensities of each cluster can be then computed. The corresponding plots for P-113 are available [here](https://gitlab.laas.fr/moma/methods/analysis/WARIO/-/tree/main/wario/demo).

## Large ensembles

The current implementation of WARIO can be run in a normal desktop computer for ensembles with $nL\lessapprox 3\cdot 10^7$. Limitations are mainly due to memory constraints. We are currently working on a more efficient implementation for large ensembles, which can be run in remote servers. Until it will be released, we can [provide guidance](mailto:javier.gonzalezdelgado@mcgill.ca) on how to run the current code in limiting settings $nL\approx 3-4\cdot 10^7$ more efficiently.


<br />
<br />
<br />
<div align="center">
<img src="images/wario.png"  width="400" height="200">
</div>
<br />
<br />
<br />
